

#include <stdio.h>
#include <unistd.h>
#include <termios.h>
#include <fcntl.h>
#include <rtems.h>
#include <rtems/score/iosh7045.h>
#include <rtems/score/sh_io.h>
#include <rtems/score/ispsh7045.h>
#include <rtems/score/isr.h>

#include <rtems/stackchk.h>
#include <rtems/cpuuse.h>
#include <rtems/monitor.h>

#include "monitor.h"
#include "clockdrv.h"

#define CONFIGURE_INIT
#include "system.h"
#include <bspIo.h>



#define EXTERN
#include "repeater.h"

#include "terminal.h"
#include "alarm_thd.h"
#include "power_thd.h"
/*
 *  Keep the names and IDs in global variables so another task can use them.
 */

rtems_id   Task_id[ 4 ];         /* array of task ids */
rtems_name Task_name[ 4 ];       /* array of task names */

/* #pragma interrupt */
rtems_isr mtu_isr(rtems_vector_number vector)
{
    write16(read16(IO_PEDR) ^ 0x0040, IO_PEDR);
}

void task_create_report(rtems_status_code status, rtems_name name)
{
    if(status == RTEMS_SUCCESSFUL)
    {
        rtems_monitor_dump_name(name);
        fflush(stdout);
        printk(" task creation success\n");
    }
    else
    {
        rtems_monitor_dump_name(name);
        fflush(stdout);
        printk(" task problem: ");
        switch(status)
        {
        case RTEMS_INVALID_NAME:
            printk("invalid name\n");
            break;
        case RTEMS_INVALID_SIZE:
            printk("stack too small\n");
            break;
        case RTEMS_INVALID_PRIORITY:
            printk("invalid priority\n");
            break;
        case RTEMS_MP_NOT_CONFIGURED:
            printk("multiprocessing not configured\n");
            break;
        case RTEMS_TOO_MANY:
            printk("too many tasks\n");
            break;
        case RTEMS_UNSATISFIED:
            printk("not enough memory for stack/FP context\n");
            break;
        default:
            printk("other\n");
        }
    }
}

void task_start_report(rtems_status_code status, rtems_name name)
{
    if(status == RTEMS_SUCCESSFUL)
    {
        rtems_monitor_dump_name(name);        
        fflush(stdout);
        printk(" task start success\n");
    }
    else
    {
        rtems_monitor_dump_name(name);
        fflush(stdout);
        printk(" task start problem: ");
        switch(status)
        {
        case RTEMS_INVALID_ADDRESS:
            printk("invalid address\n");
            break;
        case RTEMS_INVALID_ID:
            printk("invalid ID\n");
            break;
        case RTEMS_INCORRECT_STATE:
            printk("incorrect state\n");
            break;
        case RTEMS_ILLEGAL_ON_REMOTE_OBJECT:
            printk("cannot start remote task\n");
            break;
        default:
            printk("other\n");
        }
    }
}

extern void *vectab[];
rtems_isr Clock_isr(rtems_vector_number vector);



rtems_task Init(rtems_task_argument argument)
{
    rtems_status_code status;
    rtems_time_of_day time;
    unsigned int level;
    register void *vbr;
    rtems_isr_entry old_isr;
    rtems_status_code result;

    _CPU_ISR_Disable(level);
    
    printk("\n\n\n-------------------Start------------------------\n\n");
    
    asm("stc vbr, %0":"=r"(vbr));
    printf("VBR: %p vectab: %p _ISR_Vector_table: %p\n", (void*)vbr, vectab, _ISR_Vector_table);
    printf("vectab[88]: %p __ISR_Handler: %p\n", vectab[88], __ISR_Handler);
    printf("_ISR_Vector_table[88]: %p Clock_isr: %p\n", _ISR_Vector_table[88], Clock_isr);
    printf("_mtua0_isp: %p\n\n", _mtua0_isp);
    
    printf("WorkSpaceStart = %p; WorkSpace size = %08X (%d)\n",
           _Configuration_Table->work_space_start,
           _Configuration_Table->work_space_size,
           _Configuration_Table->work_space_size
           );
    
    malloc_dump();
    

/*     result = rtems_interrupt_catch(mtu_isr, 88, &old_isr); */
    
/*     if(result == RTEMS_SUCCESSFUL) */
/*         printk("Install isr success\n"); */
/*     else */
/*         printk("ISR problem: invalid %s\n", (result == RTEMS_INVALID_NUMBER ) ? */
/*                "number" : "ISR"); */
/*     vectab[88] = mtu_isr; */
    
/*     printf("VBR: %p vectab: %p _ISR_Vector_table: %p\n", (void*)vbr, vectab, _ISR_Vector_table); */
/*     printf("vectab[88]: %p __ISR_Handler: %p\n", vectab[88], __ISR_Handler); */
/*     printf("_ISR_Vector_table[88]: %p Clock_isr: %p\n", _ISR_Vector_table[88], Clock_isr); */
/*     printf("_mtua0_isp: %p mtu_isr: %p\n\n", _mtua0_isp, mtu_isr); */

    result = monitor_init();
    if(result == RTEMS_SUCCESSFUL)
        printk("Install monitor success\n");
    else
        printk("Monitor problem: invalid %s\n", (result == RTEMS_INVALID_NUMBER ) ?
               "number" : "ISR");

    rtems_monitor_init(0);    

    time.year   = 1988;
    time.month  = 12;
    time.day    = 31;
    time.hour   = 9;
    time.minute = 0;
    time.second = 0;
    time.ticks  = 0;

    status = rtems_clock_set( &time );

    Task_name[ 1 ] = rtems_build_name( 'T', 'E', 'R', 'M' );
    Task_name[ 2 ] = rtems_build_name( 'A', 'L', 'R', 'M' );
    Task_name[ 3 ] = rtems_build_name( 'M', 'O', 'D', 'M' );
    Task_name[ 4 ] = rtems_build_name( 'P', 'O', 'W', 'R' );


/*     status = rtems_task_create(Task_name[ 1 ], 1, */
/*                                RTEMS_MINIMUM_STACK_SIZE * 2,  */
/*                                RTEMS_PREEMPT | RTEMS_TIMESLICE, */
/*                                RTEMS_DEFAULT_ATTRIBUTES,  */
/*                                &Task_id[ 1 ]); */

    status = rtems_task_create(Task_name[ 2 ], 1,
                               RTEMS_MINIMUM_STACK_SIZE * 2, 
                               RTEMS_PREEMPT | RTEMS_TIMESLICE,
                               RTEMS_DEFAULT_ATTRIBUTES, 
                               &Task_id[ 2 ]);
    task_create_report(status, Task_name[2]);
    
/*     status = rtems_task_create(Task_name[ 3 ], 1, */
/*                                RTEMS_MINIMUM_STACK_SIZE * 2,  */
/*                                RTEMS_DEFAULT_MODES, */
/*                                RTEMS_DEFAULT_ATTRIBUTES,  */
/*                                &Task_id[ 3 ]); */

/*     status = rtems_task_create(Task_name[ 4 ], 1, */
/*                                RTEMS_MINIMUM_STACK_SIZE * 2,  */
/*                                RTEMS_PREEMPT | RTEMS_TIMESLICE, */
/*                                RTEMS_DEFAULT_ATTRIBUTES,  */
/*                                &Task_id[ 4 ]); */

/*     status = rtems_task_start( Task_id[ 1 ], repeater_terminal, 1 ); */
    status = rtems_task_start( Task_id[ 2 ], alarm_thread, 1 );
    task_start_report(status, Task_name[2]);
    
    malloc_dump();
    
/*     status = rtems_task_start( Task_id[ 4 ], power_thread, 1 ); */
    write16(0x00AA, IO_PEDR);

    printf("\n\n=========== Configuration registers ============\n");
    printf("BSC_BCR1   =  %04X\n", read16(BSC_BCR1));
    printf("BSC_BCR2   =  %04X\n\n", read16(BSC_BCR2));
    printf("BSC_WCR1   =  %04X\n", read16(BSC_WCR1));
    printf("BSC_WCR2   =  %04X\n\n", read16(BSC_WCR2));
    printf("BSC_DCR    =  %04X\n", read16(BSC_DCR));
    printf("BSC_RTCSR  =  %04X\n\n", read16(BSC_RTCSR));
    printf("BSC_RTCNT  =  %04X\n", read16(BSC_RTCNT));
    printf("BSC_RTCOR  =  %04X\n\n", read16(BSC_RTCOR));
    printf("PFC_PACRL1 =  %04X\n", read16(PFC_PACRL1));
    printf("PFC_PACRL2 =  %04X\n\n", read16(PFC_PACRL2));
    printf("PFC_PBCR1  =  %04X\n", read16(PFC_PBCR1));
    printf("PFC_PBCR2  =  %04X\n\n", read16(PFC_PBCR2));
    printf("PFC_PCCR   =  %04X\n", read16(PFC_PCCR));
    printf("PFC_PDCRL  =  %04X\n\n", read16(PFC_PDCRL));
    printf("PFC_IFCR   =  %04X\n", read16(PFC_IFCR));
    printf("PFC_PECR1  =  %04X\n\n", read16(PFC_PECR1));
    printf("PFC_PECR2  =  %04X\n", read16(PFC_PECR2));
    printf("PFC_PEIOR  =  %04X\n", read16(PFC_PEIOR));

    _CPU_ISR_Enable(level);
/*     asm("trapa #32"); */
    
    status = rtems_task_delete( RTEMS_SELF );
}
